

def ui2py():
    import os
    import subprocess

    list_ui_file_names = [
        'main.ui',
    ]

    cwd = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'ui')
    destination_dir = os.path.dirname(os.path.realpath(__file__))

    for ui_file_name in list_ui_file_names:
        cmd = f'pyside2-uic {os.path.join(cwd, ui_file_name)} > {os.path.join(destination_dir, ui_file_name.replace(".ui", ".py"))}'
        print(cmd)
        subprocess.call(cmd, shell=True)


if __name__ == '__main__':
    ui2py()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'main.ui'
##
## Created by: Qt User Interface Compiler version 5.14.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (QCoreApplication, QMetaObject, QObject, QPoint,
    QRect, QSize, QUrl, Qt)
from PySide2.QtGui import (QBrush, QColor, QConicalGradient, QCursor, QFont,
    QFontDatabase, QIcon, QLinearGradient, QPalette, QPainter, QPixmap,
    QRadialGradient)
from PySide2.QtWidgets import *


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(446, 186)
        sizePolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        MainWindow.setMinimumSize(QSize(426, 178))
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.pushButton_run = QPushButton(self.centralwidget)
        self.pushButton_run.setObjectName(u"pushButton_run")
        self.pushButton_run.setGeometry(QRect(335, 130, 96, 26))
        font = QFont()
        font.setPointSize(9)
        self.pushButton_run.setFont(font)
        self.pushButton_test = QPushButton(self.centralwidget)
        self.pushButton_test.setObjectName(u"pushButton_test")
        self.pushButton_test.setGeometry(QRect(230, 130, 96, 26))
        self.pushButton_test.setFont(font)
        self.groupBox = QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(u"groupBox")
        self.groupBox.setGeometry(QRect(15, 15, 416, 101))
        self.groupBox.setFont(font)
        self.pushButton_input_file = QPushButton(self.groupBox)
        self.pushButton_input_file.setObjectName(u"pushButton_input_file")
        self.pushButton_input_file.setGeometry(QRect(300, 25, 96, 26))
        self.label = QLabel(self.groupBox)
        self.label.setObjectName(u"label")
        self.label.setGeometry(QRect(10, 25, 101, 26))
        self.lineEdit_input_file = QLineEdit(self.groupBox)
        self.lineEdit_input_file.setObjectName(u"lineEdit_input_file")
        self.lineEdit_input_file.setGeometry(QRect(132, 25, 156, 26))
        self.lineEdit_n_proc = QLineEdit(self.groupBox)
        self.lineEdit_n_proc.setObjectName(u"lineEdit_n_proc")
        self.lineEdit_n_proc.setGeometry(QRect(132, 55, 156, 26))
        self.label_2 = QLabel(self.groupBox)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setGeometry(QRect(10, 55, 101, 26))
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 446, 22))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
#if QT_CONFIG(statustip)
        self.pushButton_run.setStatusTip(QCoreApplication.translate("MainWindow", u"Run simulation", None))
#endif // QT_CONFIG(statustip)
        self.pushButton_run.setText(QCoreApplication.translate("MainWindow", u"Run", None))
#if QT_CONFIG(statustip)
        self.pushButton_test.setStatusTip(QCoreApplication.translate("MainWindow", u"Make and save a template input file", None))
#endif // QT_CONFIG(statustip)
        self.pushButton_test.setText(QCoreApplication.translate("MainWindow", u"Make a Temp.", None))
        self.groupBox.setTitle(QCoreApplication.translate("MainWindow", u"Inputs", None))
        self.pushButton_input_file.setText(QCoreApplication.translate("MainWindow", u"Select", None))
        self.label.setText(QCoreApplication.translate("MainWindow", u"Input File", None))
        self.label_2.setText(QCoreApplication.translate("MainWindow", u"No. of Processes", None))
    # retranslateUi




"""SfePrapy CLI Help.
Usage:
    sfeprapy mcs0 gui
    sfeprapy mcs0 run [-p=<int>] <file_name>
    sfeprapy mcs0 figure <file_name>
    sfeprapy mcs0 template <file_name>
    sfeprapy distfit [--data_t=<int>] [--dist_g=<int>] <file_name>

Options:
    --data_t=<int>  an integer indicating data type:
                    0   (default) samples only, a single column data.
                    1   PDF, two columns data containing x, y, without heading.
                    2   CDF, two columns data containing x, y, without heading.
    --dist_g=<int>  an integer indicating what distribution group to be used for fitting the data:
                    0   fit to all available distributions.
                    1   (default) fit to common distribution types.
    -p=<int>        to define number of processes for MCS, positive integer only.
    -h --help       to show this message.

Commands:
    mcs0 gui        Experimental GUI version of sfeprapy.
    mcs0 run        Monte Carlo Simulation to solve equivalent time exposure in ISO 834 fire, method 0.
    mcs0 figure     produce figure from the output file <file_name>.
    mcs0 template   save example input file to <file_name>.
"""

from docopt import docopt


def main():
    import os

    arguments = docopt(__doc__)

    if arguments["<file_name>"]:
        arguments["<file_name>"] = os.path.realpath(arguments["<file_name>"])

    if arguments["mcs0"]:

        if arguments["gui"]:
            from sfeprapy.gui.__main__ import main as main_
            main_()
            return 0

        elif arguments["figure"]:
            from sfeprapy.mcs0.__main__ import save_figure as mcs0_figure

            mcs0_figure(fp_mcs0_out=arguments["<file_name>"])

        elif arguments["template"]:
            from sfeprapy.mcs0 import EXAMPLE_INPUT_CSV

            with open(arguments["<file_name>"], "w+") as f:
                f.write(EXAMPLE_INPUT_CSV)

        else:
            from sfeprapy.mcs0.__main__ import main as mcs0

            fp_mcs_in = arguments["<file_name>"]
            n_threads = arguments["-p"] or 2
            mcs0(fp_mcs_in=fp_mcs_in, n_threads=int(n_threads))

    elif arguments["distfit"]:

        # Prerequisites
        from sfeprapy.func.stats_dist_fit import auto_fit_2

        # Default values
        data_type = arguments["--data_t"] or 2
        distribution_list = arguments["--dist_g"] or 1

        # Main
        auto_fit_2(
            data_type=int(data_type),
            distribution_list=int(distribution_list),
            data=arguments["<file_name>"],
        )

    else:
        print("instruction unclear.")



import sys

from PySide2 import QtWidgets

from sfeprapy.gui.layout.main import Ui_MainWindow
from sfeprapy.mcs0 import EXAMPLE_INPUT_CSV
from sfeprapy.mcs0.__main__ import main as mcs0


class MainWindow(QtWidgets.QMainWindow):

    def __init__(self):
        super().__init__()  # inherit base class

        self.ui = Ui_MainWindow()  # instantiate ui
        self.ui.setupUi(self)  # set up ui

        self.setWindowTitle('SfePrapy MCS0')

        # set ui elements initial status
        self.ui.lineEdit_input_file.setReadOnly(True)

        # set ui elements connections
        self.ui.pushButton_input_file.clicked.connect(self.select_input_file)
        self.ui.pushButton_test.clicked.connect(self.make_input_template)
        self.ui.pushButton_run.clicked.connect(self.run)

    def select_input_file(self):
        """select input file and copy its path to ui object"""

        # dialog to select file
        path_to_file, _ = QtWidgets.QFileDialog.getOpenFileName(
            self,
            "Select File",
            "~/",
            "Spreadsheet (*.csv *.xls *.xlsx)")

        # paste the select file path to the ui object
        self.ui.lineEdit_input_file.setText(path_to_file)

    def make_input_template(self):
        """save an input template file to a user specified location"""

        # get the user desired file path
        path_to_file, _ = QtWidgets.QFileDialog.getSaveFileName(
            parent=self,
            caption='Save Template Input File',
            dir='mcs0.csv'
        )

        # save file
        with open(path_to_file, "w+") as f:
            f.write(EXAMPLE_INPUT_CSV)

    def run(self):
        """to fetch input and run simulation"""

        # get input file path
        try:
            fp_mcs_in = self.ui.lineEdit_input_file.text()
            if len(fp_mcs_in) == 0:  # raise error if path length is zero.
                raise
        except:
            self.ui.statusbar.showMessage('Run failed, unable to fetch input file path.')
            self.repaint()
            return -1

        # get number of processes
        try:
            n_threads = int(self.ui.lineEdit_n_proc.text())
        except ValueError:
            self.ui.statusbar.showMessage('Run failed, unable to fetch number of processes.')
            self.repaint()
            return -1

        # run simulation
        mcs0(fp_mcs_in=fp_mcs_in, n_threads=int(n_threads))

    def update_status_bar_text(self, text: str):
        self.ui.statusbar.showMessage(text=text)


def main(app_window=None):
    """main function to fire up the application cycle"""

    app = QtWidgets.QApplication(sys.argv)
    if app_window is None:
        window = MainWindow()
    else:
        window = app_window()
    window.show()
    sys.exit(app.exec_())


if __name__ == "__main__":
    """update ui files and run the application. this is used only for testing"""
    from sfeprapy.gui.layout.ui2py import ui2py
    ui2py()
    main()



import copy
import json
import os
import time
from tkinter import filedialog, Tk, StringVar
from typing import Union, Callable

import pandas as pd
from tqdm import tqdm


class MCS:
    DEFAULT_TEMP_FOLDER_NAME = "mcs.out"
    DEFAULT_MCS_OUTPUT_FILE_NAME = "mcs.out.csv"
    DEFAULT_CONFIG_FILE_NAME = "config.json"
    DEFAULT_CONFIG = dict(n_threads=1)

    def __init__(self):
        self._path_wd = None

        self._dict_master_input = None
        self._dict_config = None

        self._func_mcs_gen = None
        self._func_mcs_calc = None
        self._func_mcs_calc_mp = None
        self._func_mcs_post_1 = None

        self._df_mcs_out = None

    @property
    def path_wd(self) -> str:
        return self._path_wd

    @property
    def input(self) -> dict:
        return self._dict_master_input

    @property
    def config(self) -> dict:
        return self._dict_config

    @property
    def func_mcs_gen(self) -> Callable:
        return self._func_mcs_gen

    @property
    def func_mcs_calc(self) -> Callable:
        return self._func_mcs_calc

    @property
    def func_mcs_calc_mp(self) -> Callable:
        return self._func_mcs_calc_mp

    @property
    def func_mcs_post(self) -> Callable:
        return self._func_mcs_post_1

    @property
    def mcs_out(self) -> pd.DataFrame:
        return self._df_mcs_out

    @path_wd.setter
    def path_wd(self, p_):
        assert os.path.isdir(p_)
        self._path_wd = p_

    @input.setter
    def input(self, dict_: dict):
        self._dict_master_input = dict_

    @config.setter
    def config(self, dict_config: dict):
        self._dict_config = dict_config

    @func_mcs_gen.setter
    def func_mcs_gen(self, func_mcs_gen: Callable):
        self._func_mcs_gen = func_mcs_gen

    @func_mcs_calc.setter
    def func_mcs_calc(self, func_mcs_calc: Callable):
        self._func_mcs_calc = func_mcs_calc

    @func_mcs_calc_mp.setter
    def func_mcs_calc_mp(self, func_mcs_calc_mp: Callable):
        self._func_mcs_calc_mp = func_mcs_calc_mp

    @func_mcs_post.setter
    def func_mcs_post(self, func_mcs_post: Callable):
        self._func_mcs_post_1 = func_mcs_post

    @mcs_out.setter
    def mcs_out(self, df_out: pd.DataFrame):
        self._df_mcs_out = df_out

    def define_problem(
        self,
        data: Union[str, pd.DataFrame, dict] = None,
        config: dict = None,
        path_wd: str = None,
    ):

        # to get problem definition: try to parse from csv/xls/xlsx

        if data is None:
            fp = os.path.realpath(self._get_file_path_gui())
            self.path_wd = os.path.dirname(fp)
            if fp.endswith(".xlsx") or fp.endswith(".xls"):
                self.input = pd.read_excel(fp).set_index("PARAMETERS").to_dict()
            elif fp.endswith(".csv"):
                self.input = pd.read_csv(fp).set_index("PARAMETERS").to_dict()
            else:
                raise ValueError("Unknown input file format.")
        elif isinstance(data, str):
            fp = data
            self.path_wd = os.path.dirname(fp)
            if fp.endswith(".xlsx") or fp.endswith(".xls"):
                self.input = pd.read_excel(fp).set_index("PARAMETERS").to_dict()
            elif fp.endswith(".csv"):
                self.input = pd.read_csv(fp).set_index("PARAMETERS").to_dict()
            else:
                raise ValueError("Unknown input file format.")
        elif isinstance(data, pd.DataFrame):
            self.input = data.to_dict()
        elif isinstance(data, dict):
            self.input = data
        else:
            raise TypeError("Unknown input data type.")

        # to get configuration: try to parse from cwd if there is any, otherwise chose default values

        if config:  # first to check whether config is provided as parameter
            self.config = config
        else:  # otherwise
            try:
                with open(
                    os.path.join(self.path_wd, self.DEFAULT_CONFIG_FILE_NAME), "r"
                ) as f:
                    self.config = json.load(f)
            except (TypeError, FileNotFoundError):
                self.config = self.DEFAULT_CONFIG

    def define_stochastic_parameter_generator(self, func):
        self.func_mcs_gen = func

    def define_calculation_routine(
        self, func_mcs, func_mcs_mp=None, func_mcs_out_post=None
    ):
        self.func_mcs_calc = func_mcs
        self.func_mcs_calc_mp = func_mcs_mp
        self.func_mcs_post = func_mcs_out_post

    def run_mcs(self):
        # Check whether required parameters are defined
        err_msg = list()
        if self._dict_master_input is None:
            err_msg.append("Problem definition is not defined.")
        if self._func_mcs_calc is None:
            err_msg.append("Monte Carlo Simulation calculation routine is not defined.")
        if self._func_mcs_gen is None:
            err_msg.append(
                "Monte Carlo Simulation stochastic parameter generator is not defined."
            )
        if len(err_msg) > 0:
            raise ValueError(r"\n".join(err_msg))

        # Prepare mcs parameter inputs
        x1 = self.input
        for case_name in list(x1.keys()):
            for param_name in list(x1[case_name].keys()):

                if ":" in param_name:
                    param_name_parent, param_name_sibling = param_name.split(":")
                    if param_name_parent not in x1[case_name]:
                        x1[case_name][param_name_parent] = dict()
                    x1[case_name][param_name_parent][param_name_sibling] = x1[
                        case_name
                    ].pop(param_name)
        # to convert all "string numbers" to float data type
        for case_name in x1.keys():
            for i, v in x1[case_name].items():
                if isinstance(v, dict):
                    for ii, vv in v.items():
                        try:
                            x1[case_name][i][ii] = float(vv)
                        except:
                            pass
                else:
                    try:
                        x1[case_name][i] = float(v)
                    except:
                        pass

        # Generate mcs parameter samples
        x2 = {k: self.func_mcs_gen(v, int(v["n_simulations"])) for k, v in x1.items()}

        # Run mcs simulation
        x3 = dict()
        for k, v in x2.items():

            x3_ = self._mcs_mp(
                self.func_mcs_calc,
                self.func_mcs_calc_mp,
                x=v,
                n_threads=self.config["n_threads"],
            )
            if self.func_mcs_post:
                x3_ = self.func_mcs_post(x3_)
            x3[k] = copy.copy(x3_)

            if self.path_wd:
                if not os.path.exists(
                    os.path.join(self.path_wd, self.DEFAULT_TEMP_FOLDER_NAME)
                ):
                    os.makedirs(
                        os.path.join(self.path_wd, self.DEFAULT_TEMP_FOLDER_NAME)
                    )
                x3_.to_csv(
                    os.path.join(
                        os.path.join(self.path_wd, self.DEFAULT_TEMP_FOLDER_NAME),
                        f"{k}.csv",
                    )
                )

        self.mcs_out = pd.concat([v for v in x3.values()])

        if self.path_wd:
            self.mcs_out.to_csv(
                os.path.join(self.path_wd, self.DEFAULT_MCS_OUTPUT_FILE_NAME),
                index=False,
            )

    @staticmethod
    def _mcs_mp(func, func_mp, x: pd.DataFrame, n_threads: int) -> pd.DataFrame:
        list_mcs_in = x.to_dict(orient="records")

        time.sleep(0.5)  # to avoid clashes between the prints and progress bar
        print("{:<24.24}: {}".format("CASE", list_mcs_in[0]["case_name"]))
        print("{:<24.24}: {}".format("NO. OF THREADS", n_threads))
        print("{:<24.24}: {}".format("NO. OF SIMULATIONS", len(x.index)))
        time.sleep(0.5)  # to avoid clashes between the prints and progress bar

        if n_threads == 1 or func_mp is None:
            mcs_out = list()
            for i in tqdm(list_mcs_in, ncols=60):
                mcs_out.append(func(**i))
        else:
            import multiprocessing as mp

            m, p = mp.Manager(), mp.Pool(n_threads, maxtasksperchild=1000)
            q = m.Queue()
            jobs = p.map_async(func_mp, [(dict_, q) for dict_ in list_mcs_in])
            n_simulations = len(list_mcs_in)
            with tqdm(total=n_simulations, ncols=60) as pbar:
                while True:
                    if jobs.ready():
                        if n_simulations > pbar.n:
                            pbar.update(n_simulations - pbar.n)
                        break
                    else:
                        if q.qsize() - pbar.n > 0:
                            pbar.update(q.qsize() - pbar.n)
                        time.sleep(1)
                p.close()
                p.join()
                mcs_out = jobs.get()
                time.sleep(0.5)

        # clean and convert results to dataframe and return
        df_mcs_out = pd.DataFrame(mcs_out)
        df_mcs_out.sort_values(
            "solver_time_equivalence_solved", inplace=True
        )  # sort base on time equivalence
        return df_mcs_out

    @staticmethod
    def _get_file_path_gui():
        root = Tk()
        root.withdraw()
        folder_path = StringVar()

        path_input_file_csv = filedialog.askopenfile(
            title="Select Input File", filetypes=[("MCS IN", [".csv", ".xlsx"])]
        )
        folder_path.set(path_input_file_csv)
        root.update()

        try:
            path_input_file_csv = os.path.realpath(path_input_file_csv.name)
            return path_input_file_csv
        except AttributeError:
            raise FileNotFoundError("File not found.")




import sys

from PySide2 import QtWidgets

from sfeprapy.gui.logic.main import MainWindow


def main(app_window=None):
    """main function to fire up the application cycle"""

    app = QtWidgets.QApplication(sys.argv)
    if app_window is None:
        window = MainWindow()
    else:
        window = app_window()
    window.show()
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()
